# -*- coding: UTF-8 -*-

class Result(object):
    """
    """

    def __init__(self, **kwargs):
        self.key = kwargs.get("key")
        self.url = kwargs.get("url")
        self.primary = kwargs.get("primary")
        self.status = kwargs.get("status")
        self.fullname = kwargs.get("fullname")
        self.parent = kwargs.get("parent")
        self._childs = None

    def __unicode__(self):
        return self.fullname

    def __str__(self):
        return u"<%s %s key=%s status=%s>" % (self.__class__.__name__,
            self.fullname.encode("utf-8", "ignore"), self.key, self.status)

    def __repr__(self):
        return str(self)

    @staticmethod
    def build_instance(**kwargs):
        RANK_MAPPER = {
            "kingdom": Kingdom,
            "phylum": Phylum,
            "class": Class,
            "order": Order,
            "family": Family,
            "genus": Genus,
            "subgenus": SubGenus,
            "species": Species,
            "subspecies": SubSpecies,
            "infraspecific": Infraspecific,
            "variety": Variety,
            "form": Form,
            "nothospecies": NothoSpecies,
        }
        return RANK_MAPPER[kwargs.get("rank")](**kwargs)


class Kingdom(Result):
    """
    """
    pass


class Phylum(Result):
    """
    """
    pass


class Family(Result):
    """
    """
    pass


class Class(Result):
    """
    """
    pass


class Order(Result):
    """
    """
    pass


class Genus(Result):
    """
    """
    pass


class SubGenus(Result):
    """
    """
    pass


class Species(Result):
    """
    """
    pass


class Infraspecific(Result):
    """
    """
    pass


class Variety(Result):
    """
    """
    pass


class Form(Result):
    """
    """
    pass


class NothoSpecies(Result):
    """
    """
    pass


class SubSpecies(Result):
    """
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import codecs
import sys

from setuptools import setup, find_packages


install_requires = ['restkit']
long_description = codecs.open('README.md', "r", "utf-8").read()

setup(
    name='gbif',
    version="0.1.0",
    description="Basic and experimental Gbif Taxon API library for Python",
    long_description=long_description,
    author="Matías Agustín Méndez",
    author_email="matagus@gmail.com",
    url="http://github.com/matagus/python-gbif",
    license='MIT',
    keywords="gbif taxon taxonomy api bio",
    platforms=["any"],
    packages=find_packages(exclude=['tests']),
    install_requires=install_requires,
    zip_safe=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.4",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
    ]
)



# -*- coding: UTF-8 -*-
"Basic and experimental Gbif Taxon API library for Python"
VERSION = (0, 1, 0)

__author__ = "Matías Agustín Méndez"
__contact__ = "matagus@gmail.com"
__homepage__ = "http://github.com/matagus/python-gbif"
__version__ = ".".join(map(str, VERSION))

import sys

from urlparse import urljoin
from xml.dom.minidom import parseString

from restkit import Resource

from gbif.models import Result


class Client(Resource):

    def __init__(self, **kwargs):
        self.base_url = "http://data.gbif.org/ws/rest/taxon/"
        super(Client, self).__init__(self.base_url,
            follow_redirect=True, max_follow_redirect=10, **kwargs)

    def search(self, name, rank=None, start=0, count=50):
        kwargs = dict(scientificname=name, startindex=start,
            maxresults=count, stylesheet="")

        if rank:
            kwargs.update(rank=rank)

        return self.get("/list/", **kwargs)

    def get_by(self, key):
        if not isinstance(key, (int, long)):
            raise TypeError("Key param must be an integer number")

        path = "/get/%d/" % key
        result_list = self.get(path, stylesheet="")
        primary_taxon = None
        for taxon in result_list:
            if taxon.primary:
                primary_taxon = taxon

        primary_taxon.url = urljoin(self.base_url, path)
        return primary_taxon

    def request(self, *args, **kwargs):
        resp = super(Client, self).request(*args, **kwargs)
        body = resp.body_string()
        dom = parseString(body)
        items = []
        concept_list = dom.getElementsByTagName("tc:TaxonConcept")
        for concept in concept_list:
            items.append(self._to_item(concept))
        return items

    def _to_item(self, xml_concept):
        key = long(xml_concept.getAttribute("gbifKey"))
        status = xml_concept.getAttribute("status")
        primary = xml_concept.getElementsByTagName("tc:primary")[0].firstChild.nodeValue == "true"
        fullname = xml_concept.getElementsByTagName("tn:nameComplete")[0].firstChild.nodeValue
        rank = xml_concept.getElementsByTagName("tn:rankString")[0].firstChild.nodeValue

        return Result.build_instance(key=key, primary=primary,
            fullname=fullname, status=status, rank=rank)

if __name__ == "__main__":
    if not sys.argv[1:]:
        print "Please provide a name to search for."
        exit(0)

    name = " ".join(sys.argv[1:])

    s = Client()
    print s.search(name)
